@@ -1632,14 +1632,16 @@ class TestCore < TestInteractive
   end
 
   def test_env_vars
-    def to_vars(lines)
-      lines.select { it.start_with?('FZF_') }.to_h do
-        key, val = it.split('=', 2)
+    def env_vars
+      return {} unless File.exist?(tempname)
+
+      File.readlines(tempname).select { it.start_with?('FZF_') }.to_h do
+        key, val = it.chomp.split('=', 2)
         [key.to_sym, val]
       end
     end
 
-    tmux.send_keys %(seq 100 | #{FZF} --multi --reverse --preview-window up,99%,noborder --preview 'env | grep ^FZF_ | sort' --no-input --bind enter:show-input+refresh-preview,space:disable-search+refresh-preview), :Enter
+    tmux.send_keys %(seq 100 | #{FZF} --multi --reverse --preview-window 0 --preview 'env | grep ^FZF_ | sort > #{tempname}' --no-input --bind enter:show-input+refresh-preview,space:disable-search+refresh-preview), :Enter
     expected = {
       FZF_TOTAL_COUNT: '100',
       FZF_MATCH_COUNT: '100',
@@ -1648,31 +1650,32 @@ class TestCore < TestInteractive
       FZF_KEY: '',
       FZF_POS: '1',
       FZF_QUERY: '',
-      FZF_PROMPT: '>',
+      FZF_POINTER: '>',
+      FZF_PROMPT: '> ',
       FZF_INPUT_STATE: 'hidden'
     }
-    tmux.until do |lines|
-      assert_equal expected, to_vars(lines).slice(*expected.keys)
+    tmux.until do
+      assert_equal expected, env_vars.slice(*expected.keys)
     end
     tmux.send_keys :Enter
-    tmux.until do |lines|
+    tmux.until do
       expected.merge!(FZF_INPUT_STATE: 'enabled', FZF_ACTION: 'show-input', FZF_KEY: 'enter')
-      assert_equal expected, to_vars(lines).slice(*expected.keys)
+      assert_equal expected, env_vars.slice(*expected.keys)
     end
     tmux.send_keys :Tab, :Tab
-    tmux.until do |lines|
+    tmux.until do
       expected.merge!(FZF_ACTION: 'toggle-down', FZF_KEY: 'tab', FZF_POS: '3', FZF_SELECT_COUNT: '2')
-      assert_equal expected, to_vars(lines).slice(*expected.keys)
+      assert_equal expected, env_vars.slice(*expected.keys)
     end
     tmux.send_keys '99'
-    tmux.until do |lines|
+    tmux.until do
       expected.merge!(FZF_ACTION: 'char', FZF_KEY: '9', FZF_QUERY: '99', FZF_MATCH_COUNT: '1', FZF_POS: '1')
-      assert_equal expected, to_vars(lines).slice(*expected.keys)
+      assert_equal expected, env_vars.slice(*expected.keys)
     end
     tmux.send_keys :Space
-    tmux.until do |lines|
+    tmux.until do
       expected.merge!(FZF_INPUT_STATE: 'disabled', FZF_ACTION: 'disable-search', FZF_KEY: 'space')
-      assert_equal expected, to_vars(lines).slice(*expected.keys)
+      assert_equal expected, env_vars.slice(*expected.keys)
     end
   end
 
@@ -1856,9 +1859,9 @@ class TestCore < TestInteractive
     line = nil
     tmux.until { |lines| line = lines.index('> 1') }
     tmux.send_keys :PgDn
-    tmux.until { |lines| assert_includes lines[line + 4], "> 5" }
+    tmux.until { |lines| assert_includes lines[line + 4], '> 5' }
     tmux.send_keys :Space
-    tmux.until { |lines| assert_includes lines[line + 2], "> 5" }
+    tmux.until { |lines| assert_includes lines[line + 2], '> 5' }
   end
 
   def test_no_input_query
@@ -1927,7 +1930,10 @@ class TestCore < TestInteractive
 
   def test_change_header_on_header_window
     tmux.send_keys %(seq 100 | #{FZF} --list-border --input-border --bind 'start:change-header(foo),space:change-header(bar)'), :Enter
-    tmux.until { |lines| assert lines.any_include?('foo') }
+    tmux.until do |lines|
+      assert lines.any_include?('100/100')
+      assert lines.any_include?('foo')
+    end
     tmux.send_keys :Space
     tmux.until { |lines| assert lines.any_include?('bar') }
   end
@@ -1936,4 +1942,97 @@ class TestCore < TestInteractive
     tmux.send_keys %(echo -en "foo\n" | fzf --read0 --no-multi-line), :Enter
     tmux.until { |lines| assert_includes lines, '> foo␊' }
   end
+
+  def test_async_transform
+    time = Time.now
+    tmux.send_keys %(
+      seq 100 | #{FZF} --style full --border --preview : \
+          --bind 'focus:bg-transform-header(sleep 0.5; echo th.)' \
+          --bind 'focus:+bg-transform-footer(sleep 0.5; echo tf.)' \
+          --bind 'focus:+bg-transform-border-label(sleep 0.5; echo tbl.)' \
+          --bind "focus:+bg-transform-preview-label(sleep 0.5; echo tpl.)" \
+          --bind 'focus:+bg-transform-input-label(sleep 0.5; echo til.)' \
+          --bind 'focus:+bg-transform-list-label(sleep 0.5; echo tll.)' \
+          --bind 'focus:+bg-transform-header-label(sleep 0.5; echo thl.)' \
+          --bind 'focus:+bg-transform-footer-label(sleep 0.5; echo tfl.)' \
+          --bind 'focus:+bg-transform-prompt(sleep 0.5; echo tp.)' \
+          --bind 'focus:+bg-transform-ghost(sleep 0.5; echo tg.)'
+    ).strip, :Enter
+    tmux.until do |lines|
+      assert lines.any_include?('100/100')
+      %w[th tf tbl tpl til tll thl tfl tp tg].each do
+        assert lines.any_include?("#{it}.")
+      end
+    end
+    elapsed = Time.now - time
+    assert elapsed < 2
+  end
+
+  def test_bg_cancel
+    tmux.send_keys %(seq 0 1 | #{FZF} --bind 'space:bg-cancel+bg-transform-header(sleep {}; echo [{}])'), :Enter
+    tmux.until { assert_equal 2, it.match_count }
+    tmux.send_keys '1'
+    tmux.until { assert_equal 1, it.match_count }
+    tmux.send_keys :Space
+    tmux.send_keys :BSpace
+    tmux.until { assert_equal 2, it.match_count }
+    tmux.send_keys :Space
+    tmux.until { |lines| assert lines.any_include?('[0]') }
+    sleep 2
+    tmux.until do |lines|
+      assert lines.any_include?('[0]')
+      refute lines.any_include?('[1]')
+    end
+  end
+
+  def test_render_order
+    tmux.send_keys %(seq 100 | #{FZF} --bind='focus:preview(echo boom)+change-footer(bam)'), :Enter
+    tmux.until { assert_equal 100, it.match_count }
+    tmux.until { assert it.any_include?('boom') }
+    tmux.until { assert it.any_include?('bam') }
+  end
+
+  def test_multi_event
+    tmux.send_keys %(seq 100 | #{FZF} --multi --bind 'multi:transform-footer:(( FZF_SELECT_COUNT )) && echo "Selected $FZF_SELECT_COUNT item(s)"'), :Enter
+    tmux.until { assert_equal 100, it.match_count }
+    tmux.send_keys :Tab
+    tmux.until { assert_equal 1, it.select_count }
+    tmux.until { assert it.any_include?('Selected 1 item(s)') }
+    tmux.send_keys :Tab
+    tmux.until { assert_equal 0, it.select_count }
+    tmux.until { refute it.any_include?('Selected') }
+  end
+
+  def test_preserve_selection_on_revision_bump
+    tmux.send_keys %(seq 100 | #{FZF} --multi --sync --query "'1" --bind 'a:select-all+change-header(pressed a),b:change-header(pressed b)+change-nth(1),c:exclude'), :Enter
+    tmux.until do
+      assert_equal 20, it.match_count
+      assert_equal 0, it.select_count
+    end
+    tmux.send_keys :a
+    tmux.until do
+      assert_equal 20, it.match_count
+      assert_equal 20, it.select_count
+      assert it.any_include?('pressed a')
+    end
+    tmux.send_keys :b
+    tmux.until do
+      assert_equal 20, it.match_count
+      assert_equal 20, it.select_count
+      refute it.any_include?('pressed a')
+      assert it.any_include?('pressed b')
+    end
+    tmux.send_keys :a
+    tmux.until do
+      assert_equal 20, it.match_count
+      assert_equal 20, it.select_count
+      assert it.any_include?('pressed a')
+      refute it.any_include?('pressed b')
+    end
+    tmux.send_keys :c
+    tmux.until do
+      assert_equal 19, it.match_count
+      assert_equal 19, it.select_count
+    end
+  end
 end
